import string, re

quad = '&nbsp;' * 4
qquad = quad * 2


"""
Class associated with HTML tags, where the normal closing tag is
assumed to exist.  Since HTML tag is made up of tag name and its options,
this class defines 2 attributes, for example
    tag = 'h1'		# the name of HTML tag
    options = {'align': 'center'}	# dictionary of HTML options
so that <h1 align="center"> is the head tag and </h1> is the tail tag.
The corresponding Tag object is usually created by
    Tag('h1 align=center')
Double quotes ("...") are automatically printed around all option
values.  Also, if option value contains whitespaces, it must be set
manually.

For convenience, prefix and suffix attributes can be used to sandwich
the output content from __call__():
    <tag option=...>prefix...suffix</tag>
Prefix and suffix are not part of HTML tag, so are not available from
head() and tail().

Usage:
>>> p = Tag('tag option1 option2=value2 ...')
>>> p.head()
'<tag option1 option2="value2" ...>'
>>> p.tail()
'</tag>'
>>> p('sss')
'<tag option1 option2="value2" ...>sss</tag>'
"""
class Tag:
    def __init__(self, argv, prefix='', suffix=''):	# Tag('tag option=...')
	argv = string.split(argv)
	self.tag = argv[0]
	self.options = {}
	self.setoptions(string.join(argv[1:]))
	self.prefix = prefix
	self.suffix = suffix

    def setoptions(self, argv):		# set options to HTML tag
	argv = string.split(argv)
	for i in argv:
	    try:
		a, b = string.split(i, '=')	# option with value
	    except ValueError:
		a, b = i, ''			# option without value
	    self.options[a] = b

    def head(self):		# return '<tag option=...>'
	s = [self.tag]
	for a, b in self.options.items():
	    if  len(b) == 0:
		s.append(a)		# keyword only
	    else:
		s.append(a + '="' + b + '"')	# "..." around option values
	return '<' + string.join(s) + '>'

    def tail(self):		# return '</tag>'
	return '</' + self.tag + '>'

    def __call__(self, s=''):	# return head() + s + tail()
	return self.head() + self.prefix + s + self.suffix + self.tail()


"""
Class associated with HTML tags, where the closing tag does not exist
or can be omitted: eg. <p>, <img>.  It is same as Tag class, except
for tail() method.
"""
class OpenTag(Tag):
    def tail(self):	# there is no end tag; so, return ''
	return ''


"""
Class implementing nested HTML tags that are opened as
    <tag1 option=...><tag2 option=...>...<tagn option=...>
and closed as
    </tagn>...</tag2></tag1>
Aside from convenience, it reduces typing errors when many HTML tags
are nested.  This is same as Tag class, except for head() and tail()
which must loop over nested tags.  Because it uses head() and tail()
of Tag object, prefix and suffix attributes are not available.

Usage:
>>> p1 = Tag('tag1 option=...')
>>> p2 = Tag('tag2 option=...')
>>> p3 = Tag('tag3 option=...')
>>> p = NestedTag(p1, p2, p3)
>>> p.head()
'<tag1 option=...><tag2 option=...><tag3 option=...>'
>>> p.tail()
'</tag3></tag2></tag1>'
>>> p('sss')
'<tag1 option=...><tag2 option=...><tag3 option=...>sss</tag3></tag2></tag1>'
"""
class NestedTag:
    def __init__(self, *argv):	# NestedTag(p1, p2, ..., pn)
	self.argv = argv

    def head(self):		# return '<tag1 ...><tag2 ...>...<tagn ...>'
	s = []
	for p in self.argv:
	    s.append(p.head())
	return string.join(s, '')

    def tail(self):		# return '</tagn>...</tag2></tag1>'
	s = []
	for p in self.argv:
	    s.append(p.tail())
	s.reverse()
	return string.join(s, '')

    def __call__(self, s=''):	# return head() + s + tail()
	return self.head() + s + self.tail()


"""
Emulation of LaTeX's \footnote{}.  A global counter is incremented,
and the footnote mark and text are stored in a global Python list to
be printed later.  It returns <a href=...> Tag object with the
footnote number in prefix attribute so that it can be referenced.

Usage:
>>> a = footnote('footnote A')
>>> b = footnote('footnote B')
>>> a()
'<a href="#Foot1">[1]</a>'
>>> b()
'<a href="#Foot2">[2]</a>'
"""
footnote_counter = 0
footnote_data = []	# [(1, 'footnote A'), (2, 'footnote B'), ...]

def footnote(s):
    global footnote_counter
    n = footnote_counter = footnote_counter + 1
    footnote_data.append((n, s))
    return Tag('a href=#Foot%d' % n, '[%d]' % n)


"""
Generate footnotes from global Python list of tuple containing
footnote marks and texts.  It uses <dl compact> list.

Usage:
>>> makefootnotes()
<dl compact>
<dt> <a name="Foot1">[1]</a> <dd> footnote A
<dt> <a name="Foot2">[2]</a> <dd> footnote B 
</dl>
"""
def makefootnotes():		# print out footnote list
    print '<dl compact>'
    for n, s in footnote_data:
	a = Tag('a name=Foot%d' % n, '[%d]' % n)
	print '<dt>', a(), '<dd>', s
    print '</dl>'


"""
Emulation of LaTeX's \section{} using HTML <h4> tag.  A global counter
is incremented and all lower level counters are reset, and the section
number and title are stored in a global Python list to be printed
later.  It prints section number and title to <stdout>, and returns <a
href=...> Tag object with the section number in prefix attribute so
that it can be referenced.  

Usage:
>>> a = section('Introduction')
<h2 option=...>1 Introduction</h2>
>>> a()
'<a href="#Sec1">1</a>'
"""
section_counter0 = section_counter1 = section_counter2 = 0
section_data = []	# [('1', 'section'), ('1.1', 'subsection'), ...]

def section(s, heading='h2'):
    global section_counter0, section_counter1, section_counter2
    section_counter0 = section_counter0 + 1
    section_counter1 = 0
    section_counter2 = 0
    n = '%d' % section_counter0		# eg. 1.2.3
    section_data.append((n, s))
    h = Tag(heading)		# <h2>
    a = Tag('a name=Sec' + n)	# <a name="Sec1.2.3">
    print h(a(n + quad + s))	# <h2><a name="Sec1.2.3">1.2.3 ...</a></h2>
    return Tag('a href=#Sec' + n, n)


"""
Emulation of LaTeX's \subsection{} using HTML <h3> tag.  This is same
as section(), except that it is below section() but above
subsubsection().

Usage:
>>> a = subsection('Introduction')
<h3 option=...>1.1 Introduction</h3>
>>> a()
'<a href="#Sec1.1">1.1</a>'
"""
def subsection(s, heading='h3'):
    global section_counter0, section_counter1, section_counter2
    section_counter1 = section_counter1 + 1
    section_counter2 = 0
    n = '%d.%d' % (section_counter0, section_counter1)	# eg. 1.2.3
    section_data.append((n, s))
    h = Tag(heading)		# <h3>
    a = Tag('a name=Sec' + n)	# <a name="Sec1.2.3">
    print h(a(n + quad + s))	# <h3><a name="Sec1.2.3">1.2.3 ...</a></h3>
    return Tag('a href=#Sec' + n, n)


"""
Emulation of LaTeX's \subsubsection{} using HTML <h4> tag.  This is same
as section(), except that it is below section() and subsection().

Usage:
>>> a = subsubsection('Introduction')
<h4 option=...>1.1.1 Introduction</h4>
>>> a()
'<a href="#Sec1.1.1">1.1.1</a>'
"""
def subsubsection(s, heading='h4'):
    global section_counter0, section_counter1, section_counter2
    section_counter2 = section_counter2 + 1
    n = '%d.%d.%d' % (section_counter0, section_counter1, section_counter2)
    section_data.append((n, s))
    h = Tag(heading)		# <h4>
    a = Tag('a name=Sec' + n)	# <a name="Sec1.2.3">
    print h(a(n + quad + s))	# <h4><a name="Sec1.2.3">1.2.3 ...</a></h4>
    return Tag('a href=#Sec' + n, n)


"""
Generate table of contents from global Python list of tuple containing
sub/sub/section numbers and titles.  It uses nested <ul> list with <a
href=...> link for the entire <li> line.

Usage:
>>> tableofcontents()
<ul>
<li> 1 Section
    <ul> 
    <li> 1.1 Subsection
	<ul>
	<li> 1.1.1 Subsubsection
	</ul>
    </ul>
</ul>
"""
def tableofcontents():
    level = 0
    print '<ul>'
    for n, s in section_data:
	dotcount = string.count(n, '.')
	if dotcount > level:		# move down
	    for i in range(level, dotcount):
		print '<ul>'
	elif dotcount < level:		# move up
	    for i in range(dotcount, level):
		print '</ul>'
	level = dotcount
	a = Tag('a href=#Sec' + n)
	print '<li>', a(n + quad + s)
    for i in range(level + 1):		# close any open <ul> list
	print '</ul>'
	    

"""
Generate document title, author, and date, similar to LaTeX's
\maketitle.  Date will be generated if missing; and, email will be
printed at the closing of document if given.  It also generates the
beginning and ending of simple HTML document, like <html>, <head>,
<title>, <body>.

Usage:
>>> p = Title('title', 'author', 'date', 'email')
>>> p.begin_document()
<html><head><title>
title
</title></head><body bgcolor="whitesmoke">
>>> p.maketitle()
<center>
    <p><big><big>title</big></big></p>
    <p><big>author</big></p>
    <p><big>date</big></p>
</center>
>>> p.makeclosing()
<p><hr></p>
This file was generated using <b>HTMLtag</b> package.<br>
author &lt;email&gt;<br>
date
>>> p.end_document()
</body></html>
"""
class Title:
    def __init__(self, title, author, date='', email=''):
	self.title = title
	self.author = author
	if date == '':		# generate date, like 'March 25, 1999'
	    import time
	    sec = time.time()
	    date = time.strftime('%B %d, %Y', time.localtime(sec))
	self.date = date
	self.email = email

    def begin_document(self, title=''):	
	print '<html><head><title>'
	if title:		# title for the browser
	    print title
	else:			# use default document title
	    print self.title
	print '</title></head><body bgcolor="whitesmoke">'

    def maketitle(self):
	p = Tag('p')
	big = Tag('big')
	pbigbig = NestedTag(p, big, big)
	pbig = NestedTag(p, big)
	
	print '<center>'
	print pbigbig(self.title)
	print pbig(self.author)
	print pbig(self.date)
	print '</center>'

    def makeclosing(self):
	print '<p><hr></p>'
	print 'This file was generated using <b>HTMLtag</b> package.<br>'
	print self.author,
	if self.email:
	    print '<a href="mailto:%s">%s</a>' % (self.email, self.email),
	print '<br>'
	print self.date

    def end_document(self):
	print '</body></html>'


"""
Escape HTML unsafe characters for math expressions
    '<', '<<', '<=', '>', '>>', '>='
surrounded by whitespace.
"""
def escapemath(s):
    s = re.sub(r'(\A|\s)(<|<<|<=|>|>>|>=)(\s|\Z)', escapemath0, s)
    return s

def escapemath0(x):
    return escapeall(x.group(0))


"""
Escape HTML unsafe characters '<', '>', '&' unconditionally.
"""
def escapeall(s):
    s = string.replace(s, '&', '&amp;')		# do this first
    s = string.replace(s, '<', '&lt;')
    s = string.replace(s, '>', '&gt;')
    return s


"""
Generate HTML table from Python list of lists, emulating LaTeX's
    \begin{tabular}{rcl...} ...  \end{tabular}.
The argument specifies column alignments: 'r' for right, 'c' for
center (default), 'l' for left, ',' for \quad, and '.' for \qquad.  If
there are more columns than the format specifiers, then the column
alignment is automatically repeated.  This class defines 3 attributes
--- table, tr, td --- which are Tag objects corresponding to the HTML
table, tr, and td elements.  The output table has borders and is
aligned along the top, because it is assumed to be text mode.

Usage:
>>> p = Tabular('rcl')		# {rcl} alignments
>>> p([['a','b','c'], ['x','y','z']])	# or p(r'a & b & c\\ x & y & z')
'<table border>
<tr valign="top">
    <td align="right">a
    <td align="center">b
    <td align="left">c
<tr valign="top">
    <td align="right">x
    <td align="center">y
    <td align="left">z
</table>'
"""
class Tabular:
    rowsep = r'\\'
    colsep = '&'

    def __init__(self, format='c'):
	self.format = format
	self.table = Tag('table border')	# for text table
	self.tr = Tag('tr valign=top')		# align row along the top
	self.td = Tag('td')

    def __call__(self, data):		# convert list of list to HTML table
	if type(data) == type(''):	# if text, convert to list of list
	    data = self.text_to_list(data)
	s = []
	s.append(self.table.head())
	for row in data:
	    s.append(self.tr.head())
	    for i in range(len(row)):
		self.td.options['align'] = 'left'
		col = row[i]
		i = i % len(self.format)	# automatic repeat
		if self.format[i] == 'r': self.td.options['align'] = 'right'
		if self.format[i] == 'c': self.td.options['align'] = 'center'
		if self.format[i] == 'l': pass
		if self.format[i] == ',': col = self.quad
		if self.format[i] == '.': col = self.qquad
		s.append(self.td(col))
	    s.append(self.tr.tail())
	s.append(self.table.tail())
	return string.join(s, '\n')

    # Convert LaTeX table, where each row is separated by r'\\' and
    # each column is separated by '&', to Python list of list so that
    # this class can be used.
    #
    def text_to_list(self, data):
	out = []
	for row in string.split(data, self.rowsep):
	    r = []
	    for col in string.split(row, self.colsep):
		col = escapemath(string.strip(col))
		r.append(col)
	    out.append(r)
	return out


"""
Generate HTML table from Python list of lists, emulating LaTeX's
    \begin{array}{rcl...} ... \end{array}.
It is same as Tabular class, except that the output table has no
borders and is centered vertically, because it is assumed to be math
mode.
"""
class Array(Tabular):
    def __init__(self, format='c'):
	self.format = format
	self.table = Tag('table')	# for math table
	self.tr = Tag('tr')		# align row along the center
	self.td = Tag('td')


"""
Converts LaTeX expression to transparent GIF image.  Essentially,
LaTeX expression is inserted into .tex file
    \documentclass[12pt]{article}
    \pagestyle{empty}	% needed to suppress page number at the bottom
    \begin{document}
	LaTeX expression here.
    \end{document}
which will be processed to .eps file using latex/dvips as usual.
Then, ImageMagick's convert (my version is 3.8.7) is used to produce
transparent .gif file.  The sequence of commands are
    latex xxx.tex
    dvips -E -o xxx.eps xxx
    convert -density 144x144 -transparent white xxx.eps xxx.gif
All output to <stdout> are redirected to <stderr>, since <stdout> is
usually for HTML output.  144 DPI is chosen after few experiments.

If run_latex=1 (default), then image file will be generated and saved
even if the file already exist.  But, if run_latex=0, then correct
image is assumed to exist (from previous run) and will not be
generated.   When conversion is finished, the name and size of GIF
file are returned as OpenTag object
    <img src=... width=... height=...>
so that it can be inserted into HTML document.

Usage:
>>> img = latex2gif('$$ ... $$')
>>> img()
'<img src=... width=... height=...>'
"""
gif_counter = 0
run_latex = 1		# run LaTeX and create GIF file
latex_template = r"""
    \documentclass{article}
    \pagestyle{empty}
    \begin{document} %s \end{document}
"""			# this is printed to .tex file for LaTeX

def latex2gif(s, dpi=144):
    import sys, os
    global gif_counter

    gif_counter = gif_counter + 1
    root, suffix = os.path.splitext(sys.argv[0])
    root = root + 'gif' + `gif_counter`		# eg. test.py -> testgif1 

    if run_latex:	# run LaTeX and create GIF image
	f = open(root + '.tex', 'w')
	f.write(latex_template % s)
	f.close()
	os.system('latex %s.tex 1>&2' % root)	# .tex -> .dvi
	os.system('dvips -E -o %s.eps %s 1>&2' % (root, root))	# .dvi -> .eps
	os.system('convert -density %dx%d -transparent white %s.eps %s.gif 1>&2'
	    % (dpi, dpi, root, root))		# .eps -> .gif

    f = os.popen('identify %s.gif' % root, 'r')
    size = string.split(f.read())[1]		# eg. 200x100+0+0
    size = re.split('\D', size)
    f.close()
    return OpenTag('img src=%s width=%s height=%s' % (root + '.gif', size[0], size[1]))


"""
Emulation of LaTeX's 
    \begin{equation} ... \end{equation}
using 2 column HTML table.  First column contains the actual equation
expression which are centered in maximum possible width=100%, and
second column contains equation number which is pushed to the right of
screen by the first column.  It returns <a href=...> Tag object with
the equation number in prefix attribute so that it can be referenced.
"""
equation_counter = 0

def equation(s):
    global equation_counter
    n = equation_counter = equation_counter + 1
    a_name = Tag('a name=Eqn' + `n`)
    table = Tag('table width=100%')
    tr = Tag('tr')
    td_center = Tag('td width=100% align=center')
    td_number = Tag('td')

    print a_name(), table.head(), tr.head()
    print td_center(s), td_number('(%d)' % n)
    print tr.tail(), table.tail()
    return Tag('a href=#Eqn' + `n`, `n`)	# <a href="#Eqn1">1</a>


